@@ -19,8 +19,17 @@ Cu.import("resource://gre/modules/ctypes.jsm");
 // When modifying the payload in incompatible ways, please bump this version number
 const PAYLOAD_VERSION = 1;
 
-const PREF_SERVER = "toolkit.telemetry.server";
-const PREF_ENABLED = "toolkit.telemetry.enabled";
+// This is the HG changeset of the Histogram.json file, used to associate
+// submitted ping data with its histogram definition (bug 832007)
+const HISTOGRAMS_FILE_VERSION = "http://10.0.0.219:8000/rev/9db46ddfb517";
+
+const PREF_BRANCH = "toolkit.telemetry.";
+const PREF_SERVER = PREF_BRANCH + "server";
+//@line 31 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
+const PREF_ENABLED = PREF_BRANCH + "enabled";
+//@line 33 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
+const PREF_PREVIOUS_BUILDID = PREF_BRANCH + "previousBuildID";
+
 // Do not gather data more than once a minute
 const TELEMETRY_INTERVAL = 60000;
 // Delay before intializing telemetry (ms)
@@ -51,8 +60,10 @@ const MEM_HISTOGRAMS = {
   "js-gc-heap": "MEMORY_JS_GC_HEAP",
   "js-compartments/system": "MEMORY_JS_COMPARTMENTS_SYSTEM",
   "js-compartments/user": "MEMORY_JS_COMPARTMENTS_USER",
+  "js-main-runtime-temporary-peak": "MEMORY_JS_MAIN_RUNTIME_TEMPORARY_PEAK",
   "explicit": "MEMORY_EXPLICIT",
-  "resident": "MEMORY_RESIDENT",
+  "resident-fast": "MEMORY_RESIDENT",
+  "vsize": "MEMORY_VSIZE",
   "storage-sqlite": "MEMORY_STORAGE_SQLITE",
   "images-content-used-uncompressed":
     "MEMORY_IMAGES_CONTENT_USED_UNCOMPRESSED",
@@ -95,60 +106,6 @@ function generateUUID() {
   return str.substring(1, str.length - 1);
 }
 
-/**
- * Gets a series of simple measurements (counters). At the moment, this
- * only returns startup data from nsIAppStartup.getStartupInfo().
- * 
- * @return simple measurements as a dictionary.
- */
-function getSimpleMeasurements() {
-  let si = Services.startup.getStartupInfo();
-
-  var ret = {
-    // uptime in minutes
-    uptime: Math.round((new Date() - si.process) / 60000)
-  }
-
-  // Look for app-specific timestamps
-  var appTimestamps = {};
-  try {
-    let o = {};
-    Cu.import("resource:///modules/TelemetryTimestamps.jsm", o);
-    appTimestamps = o.TelemetryTimestamps.get();
-  } catch (ex) {}
-
-  if (si.process) {
-    for each (let field in Object.keys(si)) {
-      if (field == "process")
-        continue;
-      ret[field] = si[field] - si.process
-    }
-
-    for (let p in appTimestamps) {
-      if (!(p in ret) && appTimestamps[p])
-        ret[p] = appTimestamps[p] - si.process;
-    }
-  }
-
-  ret.startupInterrupted = new Number(Services.startup.interrupted);
-
-  // Update debuggerAttached flag
-  let debugService = Cc["@mozilla.org/xpcom/debug;1"].getService(Ci.nsIDebug2);
-  let isDebuggerAttached = debugService.isDebuggerAttached;
-  gWasDebuggerAttached = gWasDebuggerAttached || isDebuggerAttached;
-  ret.debuggerAttached = new Number(gWasDebuggerAttached);
-
-  ret.js = Cc["@mozilla.org/js/xpc/XPConnect;1"]
-           .getService(Ci.nsIJSEngineTelemetryStats)
-           .telemetryValue;
-
-  let shutdownDuration = Services.startup.lastShutdownDuration;
-  if (shutdownDuration)
-    ret.shutdownDuration = shutdownDuration;
-
-  return ret;
-}
-
 /**
  * Read current process I/O counters.
  */            
@@ -220,12 +177,92 @@ TelemetryPing.prototype = {
   _pingsLoaded: 0,
   // The number of those requests that have actually completed.
   _pingLoadsCompleted: 0,
+  _savedProfileDirectory: null,
+  // The previous build ID, if this is the first run with a new build.
+  // Undefined if this is not the first run, or the previous build ID is unknown.
+  _previousBuildID: undefined,
+
+  /**
+   * Gets a series of simple measurements (counters). At the moment, this
+   * only returns startup data from nsIAppStartup.getStartupInfo().
+   * 
+   * @return simple measurements as a dictionary.
+   */
+  getSimpleMeasurements: function getSimpleMeasurements(forSavedSession) {
+    let si = Services.startup.getStartupInfo();
+
+    var ret = {
+      // uptime in minutes
+      uptime: Math.round((new Date() - si.process) / 60000)
+    }
+
+    // Look for app-specific timestamps
+    var appTimestamps = {};
+    try {
+      let o = {};
+      Cu.import("resource://gre/modules/TelemetryTimestamps.jsm", o);
+      appTimestamps = o.TelemetryTimestamps.get();
+    } catch (ex) {}
+    try {
+      let o = {};
+      Cu.import("resource://gre/modules/AddonManager.jsm", o);
+      ret.addonManager = o.AddonManagerPrivate.getSimpleMeasures();
+    } catch (ex) {}
+
+    if (si.process) {
+      for each (let field in Object.keys(si)) {
+        if (field == "process")
+          continue;
+        ret[field] = si[field] - si.process
+      }
+
+      for (let p in appTimestamps) {
+        if (!(p in ret) && appTimestamps[p])
+          ret[p] = appTimestamps[p] - si.process;
+      }
+    }
+
+    ret.startupInterrupted = new Number(Services.startup.interrupted);
+
+    // Update debuggerAttached flag
+    let debugService = Cc["@mozilla.org/xpcom/debug;1"].getService(Ci.nsIDebug2);
+    let isDebuggerAttached = debugService.isDebuggerAttached;
+    gWasDebuggerAttached = gWasDebuggerAttached || isDebuggerAttached;
+    ret.debuggerAttached = new Number(gWasDebuggerAttached);
+
+    ret.js = Cc["@mozilla.org/js/xpc/XPConnect;1"]
+      .getService(Ci.nsIJSEngineTelemetryStats)
+      .telemetryValue;
+
+    let shutdownDuration = Telemetry.lastShutdownDuration;
+    if (shutdownDuration)
+      ret.shutdownDuration = shutdownDuration;
+
+    let failedProfileLockCount = Telemetry.failedProfileLockCount;
+    if (failedProfileLockCount)
+      ret.failedProfileLockCount = failedProfileLockCount;
+
+    for (let ioCounter in this._startupIO)
+      ret[ioCounter] = this._startupIO[ioCounter];
+
+    let hasPingBeenSent = false;
+    try {
+      hasPingBeenSent = Telemetry.getHistogramById("TELEMETRY_SUCCESS").snapshot().sum > 0;
+    } catch(e) {
+    }
+    if (!forSavedSession || hasPingBeenSent) {
+      ret.savedPings = this._pingsLoaded;
+    }
+
+    return ret;
+  },
 
   /**
    * When reflecting a histogram into JS, Telemetry hands us an object
    * with the following properties:
    * 
-   * - min, max, histogram_type, sum: simple integers;
+   * - min, max, histogram_type, sum, sum_squares_{lo,hi}: simple integers;
+   * - log_sum, log_sum_squares: doubles;
    * - counts: array of counts for histogram buckets;
    * - ranges: array of calculated bucket sizes.
    * 
@@ -235,7 +272,10 @@ TelemetryPing.prototype = {
    *
    * Returns an object:
    * { range: [min, max], bucket_count: <number of buckets>,
-   *   histogram_type: <histogram_type>, sum: <sum>
+   *   histogram_type: <histogram_type>, sum: <sum>,
+   *   sum_squares_lo: <sum_squares_lo>,
+   *   sum_squares_hi: <sum_squares_hi>,
+   *   log_sum: <log_sum>, log_sum_squares: <log_sum_squares>,
    *   values: { bucket1: count1, bucket2: count2, ... } }
    */
   packHistogram: function packHistogram(hgram) {
@@ -248,6 +288,15 @@ TelemetryPing.prototype = {
       values: {},
       sum: hgram.sum
     };
+
+    if (hgram.histogram_type == Telemetry.HISTOGRAM_EXPONENTIAL) {
+      retgram.log_sum = hgram.log_sum;
+      retgram.log_sum_squares = hgram.log_sum_squares;
+    } else {
+      retgram.sum_squares_lo = hgram.sum_squares_lo;
+      retgram.sum_squares_hi = hgram.sum_squares_hi;
+    }
+
     let first = true;
     let last = 0;
 
@@ -332,8 +381,12 @@ TelemetryPing.prototype = {
       appBuildID: ai.appBuildID,
       appUpdateChannel: UpdateChannel.get(),
       platformBuildID: ai.platformBuildID,
+      revision: HISTOGRAMS_FILE_VERSION,
       locale: getLocale()
     };
+    if (this._previousBuildID) {
+      ret.previousBuildID = this._previousBuildID;
+    }
 
     // sysinfo fields are not always available, get what we can.
     let sysInfo = Cc["@mozilla.org/system-info;1"].getService(Ci.nsIPropertyBag2);
@@ -341,7 +394,7 @@ TelemetryPing.prototype = {
                   "device", "manufacturer", "hardware",
                   "hasMMX", "hasSSE", "hasSSE2", "hasSSE3",
                   "hasSSSE3", "hasSSE4A", "hasSSE4_1", "hasSSE4_2",
-                  "hasEDSP", "hasARMv6", "hasARMv7", "hasNEON"];
+                  "hasEDSP", "hasARMv6", "hasARMv7", "hasNEON", "isWow64"];
     for each (let field in fields) {
       let value;
       try {
@@ -381,11 +434,11 @@ TelemetryPing.prototype = {
       }
     }
 
-//@line 385 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
+//@line 440 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
     let theme = LightweightThemeManager.currentTheme;
     if (theme)
       ret.persona = theme.id;
-//@line 389 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
+//@line 444 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
 
     if (this._addons)
       ret.addons = this._addons;
@@ -410,6 +463,8 @@ TelemetryPing.prototype = {
       return;
     }
 
+    let histogram = Telemetry.getHistogramById("TELEMETRY_MEMORY_REPORTER_MS");
+    let startTime = new Date();
     let e = mgr.enumerateReporters();
     while (e.hasMoreElements()) {
       let mr = e.getNext().QueryInterface(Ci.nsIMemoryReporter);
@@ -426,6 +481,7 @@ TelemetryPing.prototype = {
       catch (e) {
       }
     }
+    histogram.add(new Date() - startTime);
   },
 
   handleMemoryReport: function handleMemoryReport(id, path, units, amount) {
@@ -478,7 +534,7 @@ TelemetryPing.prototype = {
   /** 
    * Make a copy of interesting histograms at startup.
    */
-  gatherStartupInformation: function gatherStartupInformation() {
+  gatherStartupHistograms: function gatherStartupHistograms() {
     let info = Telemetry.registeredHistograms;
     let snapshots = Telemetry.histogramSnapshots;
     for (let name in info) {
@@ -487,19 +543,24 @@ TelemetryPing.prototype = {
         Telemetry.histogramFrom("STARTUP_" + name, name);
       }
     }
-    this._slowSQLStartup = Telemetry.slowSQL;
   },
 
-  getCurrentSessionPayloadAndSlug: function getCurrentSessionPayloadAndSlug(reason) {
-    // use a deterministic url for testing.
-    let isTestPing = (reason == "test-ping");
+  /**
+   * Get the current session's payload using the provided
+   * simpleMeasurements and info, which are typically obtained by a call
+   * to |this.getSimpleMeasurements| and |this.getMetadata|,
+   * respectively.
+   */
+  assemblePayloadWithMeasurements: function assemblePayloadWithMeasurements(simpleMeasurements, info) {
     let payloadObj = {
       ver: PAYLOAD_VERSION,
-      simpleMeasurements: getSimpleMeasurements(),
+      simpleMeasurements: simpleMeasurements,
       histograms: this.getHistograms(Telemetry.histogramSnapshots),
       slowSQL: Telemetry.slowSQL,
       chromeHangs: Telemetry.chromeHangs,
-      addonHistograms: this.getAddonHistograms()
+      lateWrites: Telemetry.lateWrites,
+      addonHistograms: this.getAddonHistograms(),
+      info: info
     };
 
     if (Object.keys(this._slowSQLStartup.mainThread).length
@@ -507,32 +568,33 @@ TelemetryPing.prototype = {
       payloadObj.slowSQLStartup = this._slowSQLStartup;
     }
     
-    for (let ioCounter in this._startupIO)
-      payloadObj.simpleMeasurements[ioCounter] = this._startupIO[ioCounter];
+    return payloadObj;
+  },
 
-    let hasPingBeenSent = false;
-    try {
-      hasPingBeenSent = Telemetry.getHistogramById("TELEMETRY_SUCCESS").snapshot().sum > 0;
-    } catch(e) {
-    }
-    if (reason != "saved-session" || hasPingBeenSent) {
-      payloadObj.simpleMeasurements.savedPings = this._pingsLoaded;
-    }
+  getSessionPayload: function getSessionPayload(reason) {
+    let measurements = this.getSimpleMeasurements(reason == "saved-session");
+    let info = this.getMetadata(reason);
+    return this.assemblePayloadWithMeasurements(measurements, info);
+  },
 
-    let slug = (isTestPing ? reason : this._uuid);
-    payloadObj.info = this.getMetadata(reason);
-    return { slug: slug, payload: JSON.stringify(payloadObj) };
+  assemblePing: function assemblePing(payloadObj, reason) {
+    let slug = this._uuid;
+    return { slug: slug, reason: reason, payload: JSON.stringify(payloadObj) };
+  },
+
+  getSessionPayloadAndSlug: function getSessionPayloadAndSlug(reason) {
+    return this.assemblePing(this.getSessionPayload(reason), reason);
   },
 
   getPayloads: function getPayloads(reason) {
     function payloadIter() {
-      yield this.getCurrentSessionPayloadAndSlug(reason);
+      yield this.getSessionPayloadAndSlug(reason);
 
       while (this._pendingPings.length > 0) {
         let data = this._pendingPings.pop();
         // Send persisted pings to the test URL too.
         if (reason == "test-ping") {
-          data.slug = reason;
+          data.reason = reason;
         }
         yield data;
       }
@@ -593,6 +655,7 @@ TelemetryPing.prototype = {
       this.sendPingsFromIterator(server, reason, i);
     }
     function onError() {
+      this.savePing(data, true);
       // Notify that testing is complete, even if we didn't send everything.
       finishPings(reason);
     }
@@ -617,7 +680,9 @@ TelemetryPing.prototype = {
   },
 
   doPing: function doPing(server, ping, onSuccess, onError) {
-    let submitPath = "/submit/telemetry/" + ping.slug;
+    let submitPath = "/submit/telemetry/" + (ping.reason == "test-ping"
+                                             ? "test-ping"
+                                             : ping.slug);
     let url = server + submitPath;
     let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
                   .createInstance(Ci.nsIXMLHttpRequest);
@@ -690,6 +755,23 @@ TelemetryPing.prototype = {
    * Initializes telemetry within a timer. If there is no PREF_SERVER set, don't turn on telemetry.
    */
   setup: function setup() {
+    // Record old value and update build ID preference if this is the first
+    // run with a new build ID.
+    let previousBuildID = undefined;
+    try {
+      previousBuildID = Services.prefs.getCharPref(PREF_PREVIOUS_BUILDID);
+    } catch (e) {
+      // Preference was not set.
+    }
+    let thisBuildID = Services.appinfo.appBuildID;
+    // If there is no previousBuildID preference, this._previousBuildID remains
+    // undefined so no value is sent in the telemetry metadata.
+    if (previousBuildID != thisBuildID) {
+      this._previousBuildID = previousBuildID;
+      Services.prefs.setCharPref(PREF_PREVIOUS_BUILDID, thisBuildID);
+    }
+
+//@line 785 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
     let enabled = false; 
     try {
       enabled = Services.prefs.getBoolPref(PREF_ENABLED);
@@ -703,10 +785,10 @@ TelemetryPing.prototype = {
       Telemetry.canRecord = false;
       return;
     }
-    Services.obs.addObserver(this, "private-browsing", false);
     Services.obs.addObserver(this, "profile-before-change", false);
     Services.obs.addObserver(this, "sessionstore-windows-restored", false);
     Services.obs.addObserver(this, "quit-application-granted", false);
+//@line 804 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
     Services.obs.addObserver(this, "xul-window-visible", false);
     this._hasWindowRestoredObserver = true;
     this._hasXulWindowVisibleObserver = true;
@@ -717,23 +799,28 @@ TelemetryPing.prototype = {
     this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
     function timerCallback() {
       this._initialized = true;
+      this.loadSavedPings(false);
       this.attachObservers();
       this.gatherMemory();
+
+      Telemetry.asyncFetchTelemetryData(function () {
+      });
       delete this._timer;
     }
     this._timer.initWithCallback(timerCallback.bind(this), TELEMETRY_DELAY,
                                  Ci.nsITimer.TYPE_ONE_SHOT);
-    this.loadSavedPings(false);
   },
 
-  verifyPingChecksum: function verifyPingChecksum(ping) {
+  ensurePingChecksum: function ensurePingChecksum(ping) {
     /* A ping from the current session won't have a checksum.  */
     if (!ping.checksum) {
-      return true;
+      return;
     }
 
     let checksumNow = this.hashString(ping.payload);
-    return ping.checksum == checksumNow;
+    if (ping.checksum != checksumNow) {
+      throw new Error("Invalid ping checksum")
+    }
   },
 
   addToPendingPings: function addToPendingPings(file, stream) {
@@ -744,18 +831,16 @@ TelemetryPing.prototype = {
       stream.close();
       let ping = JSON.parse(string);
       this._pingLoadsCompleted++;
-
-      if (this.verifyPingChecksum(ping)) {
-        this._pendingPings.push(ping);
-      }
-
+      // This will throw if checksum is invalid.
+      this.ensurePingChecksum(ping);
+      this._pendingPings.push(ping);
       if (this._doLoadSaveNotifications &&
           this._pingLoadsCompleted == this._pingsLoaded) {
         Services.obs.notifyObservers(null, "telemetry-test-load-complete", null);
       }
       success = true;
     } catch (e) {
-      // An error reading the file, or an error parsing the contents.
+      // An error reading the file, or an error parsing/checksumming the contents.
       stream.close();           // close is idempotent.
       file.remove(true);
     }
@@ -790,6 +875,12 @@ TelemetryPing.prototype = {
     }
   },
 
+  testLoadHistograms: function testLoadHistograms(file, sync) {
+    this._pingsLoaded = 0;
+    this._pingLoadsCompleted = 0;
+    this.loadHistograms(file, sync);
+  },
+
   loadSavedPings: function loadSavedPings(sync) {
     let directory = this.ensurePingDirectory();
     let entries = directory.directoryEntries
@@ -867,8 +958,7 @@ TelemetryPing.prototype = {
   },
 
   ensurePingDirectory: function ensurePingDirectory() {
-    let profileDirectory = Services.dirsvc.get("ProfD", Ci.nsILocalFile);
-    let directory = profileDirectory.clone();
+    let directory = this._savedProfileDirectory.clone();
     directory.append("saved-telemetry-pings");
     try {
       directory.create(Ci.nsIFile.DIRECTORY_TYPE, RWX_OWNER);
@@ -892,7 +982,7 @@ TelemetryPing.prototype = {
   },
 
   savePendingPings: function savePendingPings() {
-    let sessionPing = this.getCurrentSessionPayloadAndSlug("saved-session");
+    let sessionPing = this.getSessionPayloadAndSlug("saved-session");
     this.savePing(sessionPing, true);
     this._pendingPings.forEach(function sppcb(e, i, a) {
                                  this.savePing(e, false);
@@ -901,7 +991,7 @@ TelemetryPing.prototype = {
   },
 
   saveHistograms: function saveHistograms(file, sync) {
-    this.savePingToFile(this.getCurrentSessionPayloadAndSlug("saved-session"),
+    this.savePingToFile(this.getSessionPayloadAndSlug("saved-session"),
                         file, sync, true);
   },
 
@@ -919,23 +1009,67 @@ TelemetryPing.prototype = {
       this._hasXulWindowVisibleObserver = false;
     }
     Services.obs.removeObserver(this, "profile-before-change");
-    Services.obs.removeObserver(this, "private-browsing");
     Services.obs.removeObserver(this, "quit-application-granted");
+//@line 1028 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
+  },
+
+  getPayload: function getPayload() {
+    // This function returns the current Telemetry payload to the caller.
+    // We only gather startup info once.
+    if (Object.keys(this._slowSQLStartup).length == 0) {
+      this.gatherStartupHistograms();
+      this._slowSQLStartup = Telemetry.slowSQL;
+    }
+    this.gatherMemory();
+    return this.getSessionPayload("gather-payload");
+  },
+
+  gatherStartup: function gatherStartup() {
+    let counters = processInfo.getCounters();
+    if (counters) {
+      [this._startupIO.startupSessionRestoreReadBytes,
+        this._startupIO.startupSessionRestoreWriteBytes] = counters;
+    }
+    this.gatherStartupHistograms();
+    this._slowSQLStartup = Telemetry.slowSQL;
+  },
+
+  enableLoadSaveNotifications: function enableLoadSaveNotifications() {
+    this._doLoadSaveNotifications = true;
+  },
+
+  setAddOns: function setAddOns(aAddOns) {
+    this._addons = aAddOns;
+  },
+
+  sendIdlePing: function sendIdlePing(aTest, aServer) {
+    if (this._isIdleObserver) {
+      idleService.removeIdleObserver(this, IDLE_TIMEOUT_SECONDS);
+      this._isIdleObserver = false;
+    }
+    if (aTest) {
+      this.send("test-ping", aServer);
+    } else if (Telemetry.canSend) {
+      this.send("idle-daily", aServer);
+    }
+  },
+
+  testPing: function testPing(server) {
+    this.sendIdlePing(true, server);
+  },
+
+  cacheProfileDirectory: function cacheProfileDirectory() {
+    this._savedProfileDirectory = Services.dirsvc.get("ProfD", Ci.nsILocalFile);
   },
 
   /**
    * This observer drives telemetry.
    */
   observe: function (aSubject, aTopic, aData) {
-    // Allows to change the server for testing
-    var server = this._server;
-
     switch (aTopic) {
-    case "Add-ons":
-      this._addons = aData;
-      break;
     case "profile-after-change":
       this.setup();
+      this.cacheProfileDirectory();
       break;
     case "profile-before-change":
       this.uninstall();
@@ -948,14 +1082,6 @@ TelemetryPing.prototype = {
         this.gatherMemory();
       }
       break;
-    case "private-browsing":
-      Telemetry.canRecord = aData == "exit";
-      if (aData == "enter") {
-        this.detachObservers()
-      } else {
-        this.attachObservers()
-      }
-      break;
     case "xul-window-visible":
       Services.obs.removeObserver(this, "xul-window-visible");
       this._hasXulWindowVisibleObserver = false;   
@@ -971,14 +1097,7 @@ TelemetryPing.prototype = {
       // Check whether debugger was attached during startup
       let debugService = Cc["@mozilla.org/xpcom/debug;1"].getService(Ci.nsIDebug2);
       gWasDebuggerAttached = debugService.isDebuggerAttached;
-      // fall through
-    case "test-gather-startup":
-      var counters = processInfo.getCounters();
-      if (counters) {  
-        [this._startupIO.startupSessionRestoreReadBytes, 
-          this._startupIO.startupSessionRestoreWriteBytes] = counters;
-      }
-      this.gatherStartupInformation();
+      this.gatherStartup();
       break;
     case "idle-daily":
       // Enqueue to main-thread, otherwise components may be inited by the
@@ -991,52 +1110,21 @@ TelemetryPing.prototype = {
         this._isIdleObserver = true;
       }).bind(this), Ci.nsIThread.DISPATCH_NORMAL);
       break;
-    case "get-payload":
-      // This handler returns the current Telemetry payload to the caller.
-      // We only gather startup info once.
-      if (Object.keys(this._slowSQLStartup).length == 0)
-        this.gatherStartupInformation();
-      this.gatherMemory();
-      let data = this.getCurrentSessionPayloadAndSlug("gather-payload");
-
-      aSubject.QueryInterface(Ci.nsISupportsString).data = data.payload;
-      break;
-    case "test-save-histograms":
-      this.saveHistograms(aSubject.QueryInterface(Ci.nsIFile), aData != "async");
-      break;
-    case "test-load-histograms":
-      this._pingsLoaded = 0;
-      this._pingLoadsCompleted = 0;
-      this.loadHistograms(aSubject.QueryInterface(Ci.nsIFile), aData != "async");
-      break;
-    case "test-enable-load-save-notifications":
-      this._doLoadSaveNotifications = true;
-      break;
-    case "test-ping":
-      server = aData;
-      // fall through
     case "idle":
-      if (this._isIdleObserver) {
-        idleService.removeIdleObserver(this, IDLE_TIMEOUT_SECONDS);
-        this._isIdleObserver = false;
-      }
-      if (aTopic == "test-ping") {
-        this.send("test-ping", server);
-      }
-      else if (Telemetry.canSend && aTopic == "idle") {
-        this.send("idle-daily", server);
-      }
+      this.sendIdlePing(false, this._server);
       break;
     case "quit-application-granted":
       if (Telemetry.canSend) {
         this.savePendingPings();
       }
       break;
+
+//@line 1159 "/home/glazou/trees/official1.7/toolkit/components/telemetry/TelemetryPing.js"
     }
   },
 
   classID: Components.ID("{55d6a5fa-130e-4ee6-a158-0133af3b86ba}"),
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsITelemetryPing, Ci.nsIObserver]),
 };
 
 this.NSGetFactory = XPCOMUtils.generateNSGetFactory([TelemetryPing]);
