@@ -34,14 +34,6 @@ function WebappsRegistry() {
 WebappsRegistry.prototype = {
   __proto__: DOMRequestIpcHelper.prototype,
 
-  // Hosted apps can't be trusted or certified, so just check that the
-  // manifest doesn't ask for those.
-  checkAppStatus: function(aManifest) {
-    let manifestStatus = aManifest.type || "web";
-    return (Services.prefs.getBoolPref("dom.mozApps.dev_mode") ||
-            manifestStatus === "web");
-  },
-
   receiveMessage: function(aMessage) {
     let msg = aMessage.json;
     if (msg.oid != this._id)
@@ -53,6 +45,8 @@ WebappsRegistry.prototype = {
     switch (aMessage.name) {
       case "Webapps:Install:Return:OK":
         Services.DOMRequest.fireSuccess(req, createApplicationObject(this._window, app));
+        cpmm.sendAsyncMessage("Webapps:Install:Return:Ack",
+                              { manifestURL : app.manifestURL });
         break;
       case "Webapps:Install:Return:KO":
         Services.DOMRequest.fireError(req, msg.error || "DENIED");
@@ -80,73 +74,85 @@ WebappsRegistry.prototype = {
     return uri.prePath;
   },
 
-  _validateScheme: function(aURL) {
-    let scheme = Services.io.newURI(aURL, null, null).scheme;
-    if (scheme != "http" && scheme != "https") {
+  _validateURL: function(aURL) {
+    let uri;
+    let res;
+    try {
+      uri = Services.io.newURI(aURL, null, null);
+      if (uri.schemeIs("http") || uri.schemeIs("https")) {
+        res = uri.spec;
+      }
+    } catch(e) {
       throw new Components.Exception(
-        "INVALID_URL_SCHEME: '" + scheme + "'; must be 'http' or 'https'",
+        "INVALID_URL: '" + aURL, Cr.NS_ERROR_FAILURE
+      );
+    }
+
+    // The scheme is incorrect, throw an exception.
+    if (!res) {
+      throw new Components.Exception(
+        "INVALID_URL_SCHEME: '" + uri.scheme + "'; must be 'http' or 'https'",
         Cr.NS_ERROR_FAILURE
       );
     }
+    return uri.spec;
+  },
+
+  // Checks that we run as a foreground page, and fire an error on the
+  // DOM Request if we aren't.
+  _ensureForeground: function(aRequest) {
+    let docShell = this._window.QueryInterface(Ci.nsIInterfaceRequestor)
+                               .getInterface(Ci.nsIWebNavigation)
+                               .QueryInterface(Ci.nsIDocShell);
+    if (docShell.isActive) {
+      return true;
+    }
+
+    let runnable = {
+      run: function run() {
+        Services.DOMRequest.fireError(aRequest, "BACKGROUND_APP");
+      }
+    }
+    Services.tm.currentThread.dispatch(runnable,
+                                       Ci.nsIThread.DISPATCH_NORMAL);
+    return false;
   },
 
   // mozIDOMApplicationRegistry implementation
 
   install: function(aURL, aParams) {
-    this._validateScheme(aURL);
+    let uri = this._validateURL(aURL);
+
+    let request = this.createRequest();
+
+    if (!this._ensureForeground(request)) {
+      return request;
+    }
 
     let installURL = this._window.location.href;
-    let installOrigin = this._getOrigin(installURL);
-    let request = this.createRequest();
     let requestID = this.getRequestId(request);
-    let xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
-    xhr.open("GET", aURL, true);
-    xhr.channel.loadFlags |= Ci.nsIRequest.VALIDATE_ALWAYS;
+    let receipts = (aParams && aParams.receipts &&
+                    Array.isArray(aParams.receipts)) ? aParams.receipts
+                                                     : [];
+    let categories = (aParams && aParams.categories &&
+                      Array.isArray(aParams.categories)) ? aParams.categories
+                                                         : [];
 
-    xhr.addEventListener("load", (function() {
-      if (xhr.status == 200) {
-        let manifest;
-        try {
-          manifest = JSON.parse(xhr.responseText, installOrigin);
-        } catch (e) {
-          Services.DOMRequest.fireError(request, "MANIFEST_PARSE_ERROR");
-          Cu.reportError("Error installing app from: " + installOrigin + ": " + "MANIFEST_PARSE_ERROR");
-          return;
-        }
-
-        if (!AppsUtils.checkManifest(manifest, installOrigin)) {
-          Services.DOMRequest.fireError(request, "INVALID_MANIFEST");
-          Cu.reportError("Error installing app from: " + installOrigin + ": " + "INVALID_MANIFEST");
-        } else if (!this.checkAppStatus(manifest)) {
-          Services.DOMRequest.fireError(request, "INVALID_SECURITY_LEVEL");
-          Cu.reportError("Error installing app, '" + manifest.name + "': " + "INVALID_SECURITY_LEVEL");
-        } else {
-          let receipts = (aParams && aParams.receipts && Array.isArray(aParams.receipts)) ? aParams.receipts : [];
-          let categories = (aParams && aParams.categories && Array.isArray(aParams.categories)) ? aParams.categories : [];
-          let etag = xhr.getResponseHeader("Etag");
-          cpmm.sendAsyncMessage("Webapps:Install", { app: { installOrigin: installOrigin,
-                                                            origin: this._getOrigin(aURL),
-                                                            manifestURL: aURL,
-                                                            manifest: manifest,
-                                                            etag: etag,
-                                                            receipts: receipts,
-                                                            categories: categories },
-                                                            from: installURL,
-                                                            oid: this._id,
-                                                            requestID: requestID });
-        }
-      } else {
-        Services.DOMRequest.fireError(request, "MANIFEST_URL_ERROR");
-        Cu.reportError("Error installing app from: " + installOrigin + ": " + "MANIFEST_URL_ERROR");
-      }
-    }).bind(this), false);
-
-    xhr.addEventListener("error", (function() {
-      Services.DOMRequest.fireError(request, "NETWORK_ERROR");
-      Cu.reportError("Error installing app from: " + installOrigin + ": " + "NETWORK_ERROR");
-    }).bind(this), false);
-
-    xhr.send(null);
+    let principal = this._window.document.nodePrincipal;
+    cpmm.sendAsyncMessage("Webapps:Install",
+                          { app: {
+                              installOrigin: this._getOrigin(installURL),
+                              origin: this._getOrigin(uri),
+                              manifestURL: uri,
+                              receipts: receipts,
+                              categories: categories
+                            },
+                            from: installURL,
+                            oid: this._id,
+                            requestID: requestID,
+                            appId: principal.appId,
+                            isBrowser: principal.isInBrowserElement
+                          });
     return request;
   },
 
@@ -180,6 +186,10 @@ WebappsRegistry.prototype = {
   },
 
   get mgmt() {
+    if (!this.hasMgmtPrivilege) {
+      return null;
+    }
+
     if (!this._mgmt)
       this._mgmt = new WebappsApplicationMgmt(this._window);
     return this._mgmt;
@@ -194,65 +204,39 @@ WebappsRegistry.prototype = {
   // mozIDOMApplicationRegistry2 implementation
 
   installPackage: function(aURL, aParams) {
-    let installURL = this._window.location.href;
-    let installOrigin = this._getOrigin(installURL);
-    this._validateScheme(aURL);
+    let uri = this._validateURL(aURL);
 
     let request = this.createRequest();
+
+    if (!this._ensureForeground(request)) {
+      return request;
+    }
+
+    let installURL = this._window.location.href;
     let requestID = this.getRequestId(request);
-
     let receipts = (aParams && aParams.receipts &&
-                    Array.isArray(aParams.receipts)) ? aParams.receipts : [];
+                    Array.isArray(aParams.receipts)) ? aParams.receipts
+                                                     : [];
     let categories = (aParams && aParams.categories &&
-                      Array.isArray(aParams.categories)) ? aParams.categories : [];
-    let xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
-    xhr.open("GET", aURL, true);
-    xhr.channel.loadFlags |= Ci.nsIRequest.VALIDATE_ALWAYS;
+                      Array.isArray(aParams.categories)) ? aParams.categories
+                                                         : [];
 
-    xhr.addEventListener("load", (function() {
-      if (xhr.status == 200) {
-        let manifest;
-        try {
-          manifest = JSON.parse(xhr.responseText, installOrigin);
-        } catch(e) {
-          Services.DOMRequest.fireError(request, "MANIFEST_PARSE_ERROR");
-          return;
-        }
-        if (!(AppsUtils.checkManifest(manifest, installOrigin) &&
-              manifest.package_path)) {
-          Services.DOMRequest.fireError(request, "INVALID_MANIFEST");
-        } else {
-          if (!this.checkAppStatus(manifest)) {
-            Services.DOMRequest.fireError(request, "INVALID_SECURITY_LEVEL");
-          } else {
-            let receipts = (aParams && aParams.receipts && Array.isArray(aParams.receipts)) ? aParams.receipts : [];
-            let categories = (aParams && aParams.categories && Array.isArray(aParams.categories)) ? aParams.categories : [];
-            let etag = xhr.getResponseHeader("Etag");
-            cpmm.sendAsyncMessage("Webapps:InstallPackage", { app: {
-                                                              installOrigin: installOrigin,
-                                                              origin: this._getOrigin(aURL),
-                                                              manifestURL: aURL,
-                                                              updateManifest: manifest,
-                                                              etag: etag,
-                                                              receipts: receipts,
-                                                              categories: categories },
-                                                              from: installURL,
-                                                              oid: this._id,
-                                                              requestID: requestID,
-                                                              isPackage: true });
-          }
-        }
-      }
-      else {
-        Services.DOMRequest.fireError(request, "MANIFEST_URL_ERROR");
-      }
-    }).bind(this), false);
-
-    xhr.addEventListener("error", (function() {
-      Services.DOMRequest.fireError(request, "NETWORK_ERROR");
-    }).bind(this), false);
-
-    xhr.send(null);
+    let principal = this._window.document.nodePrincipal;
+    cpmm.sendAsyncMessage("Webapps:InstallPackage",
+                          { app: {
+                              installOrigin: this._getOrigin(installURL),
+                              origin: this._getOrigin(uri),
+                              manifestURL: uri,
+                              receipts: receipts,
+                              categories: categories
+                            },
+                            from: installURL,
+                            oid: this._id,
+                            requestID: requestID,
+                            isPackage: true,
+                            appId: principal.appId,
+                            isBrowser: principal.isInBrowserElement
+                          });
     return request;
   },
 
@@ -263,26 +247,31 @@ WebappsRegistry.prototype = {
                               "Webapps:GetSelf:Return:OK",
                               "Webapps:CheckInstalled:Return:OK" ]);
 
-    let util = this._window.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowUtils);
+    let util = this._window.QueryInterface(Ci.nsIInterfaceRequestor)
+                           .getInterface(Ci.nsIDOMWindowUtils);
     this._id = util.outerWindowID;
     cpmm.sendAsyncMessage("Webapps:RegisterForMessages",
                           ["Webapps:Install:Return:OK"]);
+
+    let principal = aWindow.document.nodePrincipal;
+    let perm = Services.perms
+               .testExactPermissionFromPrincipal(principal, "webapps-manage");
+
+    // Only pages with the webapps-manage permission set can get access to
+    // the mgmt object.
+    this.hasMgmtPrivilege = perm == Ci.nsIPermissionManager.ALLOW_ACTION;
   },
 
   classID: Components.ID("{fff440b3-fae2-45c1-bf03-3b5a2e432270}"),
 
   QueryInterface: XPCOMUtils.generateQI([Ci.mozIDOMApplicationRegistry,
-//@line 286 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
-                                         Ci.mozIDOMApplicationRegistry2,
-//@line 288 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
+//@line 273 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
                                          Ci.nsIDOMGlobalPropertyInitializer]),
 
   classInfo: XPCOMUtils.generateCI({classID: Components.ID("{fff440b3-fae2-45c1-bf03-3b5a2e432270}"),
                                     contractID: "@mozilla.org/webapps;1",
                                     interfaces: [Ci.mozIDOMApplicationRegistry,
-//@line 304 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
-                                                 Ci.mozIDOMApplicationRegistry2,
-//@line 306 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
+//@line 283 "/home/glazou/trees/official1.7/dom/apps/src/Webapps.js"
                                                  ],
                                     flags: Ci.nsIClassInfo.DOM_OBJECT,
                                     classDescription: "Webapps Registry"})
@@ -322,6 +311,39 @@ DOMError.prototype = {
   * mozIDOMApplication object
   */
 
+// A simple cache for the wrapped manifests.
+let manifestCache = {
+  _cache: { },
+
+  // Gets an entry from the cache, and populates the cache if needed.
+  get: function mcache_get(aManifestURL, aManifest, aWindow, aInnerWindowID) {
+    if (!(aManifestURL in this._cache)) {
+      this._cache[aManifestURL] = { };
+    }
+
+    let winObjs = this._cache[aManifestURL];
+    if (!(aInnerWindowID in winObjs)) {
+      winObjs[aInnerWindowID] = ObjectWrapper.wrap(aManifest, aWindow);
+    }
+
+    return winObjs[aInnerWindowID];
+  },
+
+  // Invalidates an entry in the cache.
+  evict: function mcache_evict(aManifestURL, aInnerWindowID) {
+    if (aManifestURL in this._cache) {
+      let winObjs = this._cache[aManifestURL];
+      if (aInnerWindowID in winObjs) {
+        delete winObjs[aInnerWindowID];
+      }
+
+      if (Object.keys(winObjs).length == 0) {
+        delete this._cache[aManifestURL];
+      }
+    }
+  }
+};
+
 function createApplicationObject(aWindow, aApp) {
   let app = Cc["@mozilla.org/webapps/application;1"].createInstance(Ci.mozIDOMApplication);
   app.wrappedJSObject.init(aWindow, aApp);
@@ -348,6 +370,8 @@ WebappsApplication.prototype = {
     this.removable = aApp.removable;
     this.lastUpdateCheck = aApp.lastUpdateCheck ? aApp.lastUpdateCheck
                                                 : Date.now();
+    this.updateTime = aApp.updateTime ? aApp.updateTime
+                                      : aApp.installTime;
     this.progress = NaN;
     this.downloadAvailable = aApp.downloadAvailable;
     this.downloading = aApp.downloading;
@@ -362,19 +386,23 @@ WebappsApplication.prototype = {
 
     this._downloadError = null;
 
-    this.initHelper(aWindow, ["Webapps:Uninstall:Return:OK",
-                              "Webapps:Uninstall:Return:KO",
-                              "Webapps:OfflineCache",
+    this.initHelper(aWindow, ["Webapps:OfflineCache",
                               "Webapps:CheckForUpdate:Return:OK",
                               "Webapps:CheckForUpdate:Return:KO",
+                              "Webapps:Launch:Return:KO",
                               "Webapps:PackageEvent"]);
+
     cpmm.sendAsyncMessage("Webapps:RegisterForMessages",
-                          ["Webapps:Uninstall:Return:OK", "Webapps:OfflineCache",
-                           "Webapps:PackageEvent"]);
+                          ["Webapps:OfflineCache",
+                           "Webapps:PackageEvent",
+                           "Webapps:CheckForUpdate:Return:OK"]);
   },
 
   get manifest() {
-    return this.manifest = ObjectWrapper.wrap(this._manifest, this._window);
+    return manifestCache.get(this.manifestURL,
+                             this._manifest,
+                             this._window,
+                             this.innerWindowID);
   },
 
   get updateManifest() {
@@ -438,6 +466,7 @@ WebappsApplication.prototype = {
 
   checkForUpdate: function() {
     let request = this.createRequest();
+
     cpmm.sendAsyncMessage("Webapps:CheckForUpdate",
                           { manifestURL: this.manifestURL,
                             oid: this._id,
@@ -451,18 +480,11 @@ WebappsApplication.prototype = {
                                               manifestURL: this.manifestURL,
                                               startPoint: aStartPoint || "",
                                               oid: this._id,
+                                              timestamp: Date.now(),
                                               requestID: this.getRequestId(request) });
     return request;
   },
 
-  uninstall: function() {
-    let request = this.createRequest();
-    cpmm.sendAsyncMessage("Webapps:Uninstall", { origin: this.origin,
-                                                 oid: this._id,
-                                                 requestID: this.getRequestId(request) });
-    return request;
-  },
-
   clearBrowserData: function() {
     let browserChild =
       BrowserElementPromptService.getBrowserElementChildForWindow(this._window);
@@ -474,8 +496,11 @@ WebappsApplication.prototype = {
   uninit: function() {
     this._onprogress = null;
     cpmm.sendAsyncMessage("Webapps:UnregisterForMessages",
-                          ["Webapps:Uninstall:Return:OK", "Webapps:OfflineCache",
-                           "Webapps:PackageEvent"]);
+                          ["Webapps:OfflineCache",
+                           "Webapps:PackageEvent",
+                           "Webapps:CheckForUpdate:Return:OK"]);
+
+    manifestCache.evict(this.manifestURL, this.innerWindowID);
   },
 
   _fireEvent: function(aName, aHandler) {
@@ -488,22 +513,37 @@ WebappsApplication.prototype = {
   receiveMessage: function(aMessage) {
     let msg = aMessage.json;
     let req = this.takeRequest(msg.requestID);
+
+    // ondownload* callbacks should be triggered on all app instances
     if ((msg.oid != this._id || !req) &&
         aMessage.name !== "Webapps:OfflineCache" &&
-        aMessage.name !== "Webapps:PackageEvent")
+        aMessage.name !== "Webapps:PackageEvent" &&
+        aMessage.name !== "Webapps:CheckForUpdate:Return:OK")
       return;
     switch (aMessage.name) {
-      case "Webapps:Uninstall:Return:OK":
-        Services.DOMRequest.fireSuccess(req, msg.origin);
-        break;
-      case "Webapps:Uninstall:Return:KO":
-        Services.DOMRequest.fireError(req, "NOT_INSTALLED");
-        break;
       case "Webapps:Launch:Return:KO":
         Services.DOMRequest.fireError(req, "APP_INSTALL_PENDING");
         break;
-      case "Webapps:Uninstall:Return:KO":
-        Services.DOMRequest.fireError(req, "NOT_INSTALLED");
+      case "Webapps:CheckForUpdate:Return:KO":
+        Services.DOMRequest.fireError(req, msg.error);
+        break;
+      case "Webapps:CheckForUpdate:Return:OK":
+        if (msg.manifestURL != this.manifestURL)
+          return;
+
+        for (let prop in msg.app) {
+          this[prop] = msg.app[prop];
+        }
+
+        if (msg.event == "downloadapplied") {
+          this._fireEvent("downloadapplied", this._ondownloadapplied);
+        } else if (msg.event == "downloadavailable") {
+          this._fireEvent("downloadavailable", this._ondownloadavailable);
+        }
+
+        if (req) {
+          Services.DOMRequest.fireSuccess(req, this.manifestURL);
+        }
         break;
       case "Webapps:OfflineCache":
         if (msg.manifest != this.manifestURL)
@@ -511,11 +551,14 @@ WebappsApplication.prototype = {
 
         if ("installState" in msg) {
           this.installState = msg.installState;
+          this.progress = msg.progress;
           if (this.installState == "installed") {
             this._downloadError = null;
+            this.downloading = false;
             this._fireEvent("downloadsuccess", this._ondownloadsuccess);
             this._fireEvent("downloadapplied", this._ondownloadapplied);
           } else {
+            this.downloading = true;
             this._fireEvent("downloadprogress", this._onprogress);
           }
         } else if (msg.error) {
@@ -524,70 +567,51 @@ WebappsApplication.prototype = {
           this._fireEvent("downloaderror", this._ondownloaderror);
         }
         break;
-        case "Webapps:CheckForUpdate:Return:OK":
-          for (let prop in msg.app) {
-            this[prop] = msg.app[prop];
-          }
-          if (msg.event == "downloadapplied") {
-            Services.DOMRequest.fireSuccess(req, this.manifestURL);
+      case "Webapps:PackageEvent":
+        if (msg.manifestURL != this.manifestURL)
+          return;
+
+        // Set app values according to parent process results.
+        let app = msg.app;
+        this.downloading = app.downloading;
+        this.downloadAvailable = app.downloadAvailable;
+        this.downloadSize = app.downloadSize || 0;
+        this.installState = app.installState;
+        this.progress = app.progress || msg.progress || 0;
+        this.readyToApplyDownload = app.readyToApplyDownload;
+        this.updateTime = app.updateTime;
+
+        switch(msg.type) {
+          case "error":
+          case "canceled":
+            this._downloadError = msg.error;
+            this._fireEvent("downloaderror", this._ondownloaderror);
+            break;
+          case "progress":
+            this._fireEvent("downloadprogress", this._onprogress);
+            break;
+          case "installed":
+            manifestCache.evict(this.manifestURL, this.innerWindowID);
+            this._manifest = msg.manifest;
+            this._fireEvent("downloadsuccess", this._ondownloadsuccess);
             this._fireEvent("downloadapplied", this._ondownloadapplied);
-          } else if (msg.event == "downloadavailable") {
-            Services.DOMRequest.fireSuccess(req, this.manifestURL);
-            this._fireEvent("downloadavailable", this._ondownloadavailable);
-          }
-          break;
-        case "Webapps:CheckForUpdate:Return:KO":
-          Services.DOMRequest.fireError(req, msg.error);
-          break;
-        case "Webapps:PackageEvent":
-          if (msg.manifestURL != this.manifestURL)
-            return;
-          switch(msg.type) {
-            case "error":
-              this._downloadError = msg.error;
-              this._fireEvent("downloaderror", this._ondownloaderror);
-              break;
-            case "progress":
-              this.progress = msg.progress;
-              this._fireEvent("downloadprogress", this._onprogress);
-              break;
-            case "installed":
-              let app = msg.app;
-              this.progress = app.progress || 0;
-              this.downloadAvailable = app.downloadAvailable;
-              this.downloading = app.downloading;
-              this.readyToApplyDownload = app.readyToApplyDownload;
-              this.downloadSize = app.downloadSize || 0;
-              this.installState = app.installState;
+            break;
+          case "downloaded":
+            // We don't update the packaged apps manifests until they
+            // are installed or until the update is unstaged.
+            if (msg.manifest) {
+              manifestCache.evict(this.manifestURL, this.innerWindowID);
               this._manifest = msg.manifest;
-              this._fireEvent("downloadsuccess", this._ondownloadsuccess);
-              this._fireEvent("downloadapplied", this._ondownloadapplied);
-              break;
-            case "canceled":
-              app = msg.app;
-              this.progress = app.progress || 0;
-              this.downloadAvailable = app.downloadAvailable;
-              this.downloading = app.downloading;
-              this.readyToApplyDownload = app.readyToApplyDownload;
-              this.downloadSize = app.downloadSize || 0;
-              this.installState = app.installState;
-              this._downloadError = msg.error;
-              this._fireEvent("downloaderror", this._ondownloaderror);
-              break;
-            case "downloaded":
-              app = msg.app;
-              this.downloading = app.downloading;
-              this.downloadAvailable = app.downloadAvailable;
-              this.readyToApplyDownload = app.readyToApplyDownload;
-              this._fireEvent("downloadsuccess", this._ondownloadsuccess);
-              break;
-            case "applied":
-              app = msg.app;
-              this.readyToApplyDownload = app.readyToApplyDownload;
-              this._fireEvent("downloadapplied", this._ondownloadapplied);
-              break;
-          }
-          break;
+            }
+            this._fireEvent("downloadsuccess", this._ondownloadsuccess);
+            break;
+          case "applied":
+            manifestCache.evict(this.manifestURL, this.innerWindowID);
+            this._manifest = msg.manifest;
+            this._fireEvent("downloadapplied", this._ondownloadapplied);
+            break;
+        }
+        break;
     }
   },
 
@@ -606,22 +630,18 @@ WebappsApplication.prototype = {
   * mozIDOMApplicationMgmt object
   */
 function WebappsApplicationMgmt(aWindow) {
-  let principal = aWindow.document.nodePrincipal;
-  let secMan = Cc["@mozilla.org/scriptsecuritymanager;1"].getService(Ci.nsIScriptSecurityManager);
-
-  let perm = principal == secMan.getSystemPrincipal()
-               ? Ci.nsIPermissionManager.ALLOW_ACTION
-               : Services.perms.testExactPermissionFromPrincipal(principal, "webapps-manage");
-
-  //only pages with perm set can use some functions
-  this.hasPrivileges = perm == Ci.nsIPermissionManager.ALLOW_ACTION;
-
-  this.initHelper(aWindow, ["Webapps:GetAll:Return:OK", "Webapps:GetAll:Return:KO",
-                            "Webapps:Install:Return:OK", "Webapps:Uninstall:Return:OK",
+  this.initHelper(aWindow, ["Webapps:GetAll:Return:OK",
+                            "Webapps:GetAll:Return:KO",
+                            "Webapps:Uninstall:Return:OK",
+                            "Webapps:Uninstall:Broadcast:Return:OK",
+                            "Webapps:Uninstall:Return:KO",
+                            "Webapps:Install:Return:OK",
                             "Webapps:GetNotInstalled:Return:OK"]);
 
   cpmm.sendAsyncMessage("Webapps:RegisterForMessages",
-                        ["Webapps:Install:Return:OK", "Webapps:Uninstall:Return:OK"]);
+                        ["Webapps:Install:Return:OK",
+                         "Webapps:Uninstall:Return:OK",
+                         "Webapps:Uninstall:Broadcast:Return:OK"]);
 
   this._oninstall = null;
   this._onuninstall = null;
@@ -641,7 +661,9 @@ WebappsApplicationMgmt.prototype = {
     this._oninstall = null;
     this._onuninstall = null;
     cpmm.sendAsyncMessage("Webapps:UnregisterForMessages",
-                          ["Webapps:Install:Return:OK", "Webapps:Uninstall:Return:OK"]);
+                          ["Webapps:Install:Return:OK",
+                           "Webapps:Uninstall:Return:OK",
+                           "Webapps:Uninstall:Broadcast:Return:OK"]);
   },
 
   applyDownload: function(aApp) {
@@ -653,11 +675,19 @@ WebappsApplicationMgmt.prototype = {
                           { manifestURL: aApp.manifestURL });
   },
 
+  uninstall: function(aApp) {
+    dump("-- webapps.js uninstall " + aApp.manifestURL + "\n");
+    let request = this.createRequest();
+    cpmm.sendAsyncMessage("Webapps:Uninstall", { origin: aApp.origin,
+                                                 oid: this._id,
+                                                 requestID: this.getRequestId(request) });
+    return request;
+  },
+
   getAll: function() {
     let request = this.createRequest();
     cpmm.sendAsyncMessage("Webapps:GetAll", { oid: this._id,
-                                              requestID: this.getRequestId(request),
-                                              hasPrivileges: this.hasPrivileges });
+                                              requestID: this.getRequestId(request) });
     return request;
   },
 
@@ -677,27 +707,23 @@ WebappsApplicationMgmt.prototype = {
   },
 
   set oninstall(aCallback) {
-    if (this.hasPrivileges)
-      this._oninstall = aCallback;
-    else
-      throw new Components.Exception("Denied", Cr.NS_ERROR_FAILURE);
+    this._oninstall = aCallback;
   },
 
   set onuninstall(aCallback) {
-    if (this.hasPrivileges)
-      this._onuninstall = aCallback;
-    else
-      throw new Components.Exception("Denied", Cr.NS_ERROR_FAILURE);
+    this._onuninstall = aCallback;
   },
 
   receiveMessage: function(aMessage) {
     var msg = aMessage.json;
     let req = this.getRequest(msg.requestID);
-    // We want Webapps:Install:Return:OK and Webapps:Uninstall:Return:OK to be boradcasted
-    // to all instances of mozApps.mgmt
-    if (!((msg.oid == this._id && req)
-       || aMessage.name == "Webapps:Install:Return:OK" || aMessage.name == "Webapps:Uninstall:Return:OK"))
+    // We want Webapps:Install:Return:OK and Webapps:Uninstall:Broadcast:Return:OK
+    // to be boradcasted to all instances of mozApps.mgmt.
+    if (!((msg.oid == this._id && req) ||
+          aMessage.name == "Webapps:Install:Return:OK" ||
+          aMessage.name == "Webapps:Uninstall:Broadcast:Return:OK")) {
       return;
+    }
     switch (aMessage.name) {
       case "Webapps:GetAll:Return:OK":
         Services.DOMRequest.fireSuccess(req, convertAppsArray(msg.apps, this._window));
@@ -716,15 +742,27 @@ WebappsApplicationMgmt.prototype = {
           this._oninstall.handleEvent(event);
         }
         break;
-      case "Webapps:Uninstall:Return:OK":
+      case "Webapps:Uninstall:Broadcast:Return:OK":
         if (this._onuninstall) {
+          let detail = {
+            manifestURL: msg.manifestURL,
+            origin: msg.origin
+          };
           let event = new this._window.MozApplicationEvent("applicationuninstall",
-                           { application : createApplicationObject(this._window, { origin: msg.origin }) });
+                           { application : createApplicationObject(this._window, detail) });
           this._onuninstall.handleEvent(event);
         }
         break;
+      case "Webapps:Uninstall:Return:OK":
+        Services.DOMRequest.fireSuccess(req, msg.origin);
+        break;
+      case "Webapps:Uninstall:Return:KO":
+        Services.DOMRequest.fireError(req, "NOT_INSTALLED");
+        break;
+    }
+    if (aMessage.name !== "Webapps:Uninstall:Broadcast:Return:OK") {
+      this.removeRequest(msg.requestID);
     }
-    this.removeRequest(msg.requestID);
   },
 
   classID: Components.ID("{8c1bca96-266f-493a-8d57-ec7a95098c15}"),
